package edu.toronto.cs.algorithms;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.BitSet;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import edu.toronto.cs.parsers.LocalIrrelevanceProofParser;
import edu.toronto.cs.util.StopWatch;

public class LocalIrrelevance {

	public static DefaultHandler handler;
	public static SAXParserFactory factory = SAXParserFactory.newInstance();

	public static BitSet checkLocalIrrelevance(File proofFile, File modelFile, File propertyFile, ArrayList<Integer> variables) throws IOException {
		StopWatch stopWatch = new StopWatch();
		try {

			handler = new LocalIrrelevanceProofParser(variables);

			SAXParser saxParser = factory.newSAXParser();
	
			stopWatch.reset();
			saxParser.parse(proofFile, handler);
			stopWatch.pause();
			
			System.out.println("Generate Local Irrel results: " + stopWatch);

		} catch (SAXException ex) {
			System.err.println("SAXException " + ex.getMessage());
			System.exit(2);
		} catch (ParserConfigurationException ex) {
			System.err.println("ParserCopnfigurationException " + ex.getMessage());
			System.exit(2);
		} catch (IOException ex) {
			System.err.println("IOException " + ex.getMessage());
			System.exit(2);
		}

		// first half has model labels, second half has property labels
		// to get final result, must AND both halves
		BitSet results = ((LocalIrrelevanceProofParser)handler).getResults();
		
		BitSet r1 = results.get(0, variables.size());
		BitSet r2 = results.get(variables.size(), 2 * variables.size());
		r1.and(r2);
		
		return r1; 
	}
}
