package edu.toronto.cs.parsers;

import java.util.ArrayList;
import java.util.BitSet;
import java.util.Collections;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import edu.toronto.cs.algorithms.Peripherality;
import edu.toronto.cs.info.Clause;
import edu.toronto.cs.util.Constants;

public class PeripheralityProofParser extends DefaultHandler {

	public static ArrayList<Integer> currentRootClause;
	public static int currentRootIndex;
	public static int currentRootMP;

	public static int currentChildren;
	public static int lastIndex;

	public static int currentDerivedIndex;
	public static ArrayList<Integer> currentChainId;
	public static ArrayList<Integer> currentChainVar;

	public static boolean sign;
	public static int index;

	public PeripheralityProofParser(int lastIndex){
		super();
		this.lastIndex = lastIndex;
	}

	public void startElement(String namespaceURI, String sName, String qName, Attributes attrs) throws SAXException {
		String eName = sName;

		if ("".equals(eName))
			eName = qName;

		if (eName.equals(Constants.ROOT)) {
			currentRootIndex = Integer.parseInt(attrs.getValue("i"));
			currentRootMP = Integer.parseInt(attrs.getValue("o"));
			currentChildren = Integer.parseInt(attrs.getValue("c"));

			currentRootClause = new ArrayList<Integer>();

//			System.out.println(" R " + currentRootIndex);
		} else if (eName.equals(Constants.VARIABLE)) {
			sign = Boolean.parseBoolean(attrs.getValue("s"));
			index = Integer.parseInt(attrs.getValue("i"));

			currentRootClause.add(sign ? (-1 * index) : index);
		}
		else if (eName.equals(Constants.DERIVED)) {
			currentDerivedIndex = Integer.parseInt(attrs.getValue("i"));
			
			currentChainId = new ArrayList<Integer>();
			currentChainVar = new ArrayList<Integer>();

			currentChildren = Integer.parseInt(attrs.getValue("c"));

//			System.out.println(" D " + currentDerivedIndex);
		}
		else if (eName.equals(Constants.PARENT)) {
			currentChainId.add(Integer.parseInt(attrs.getValue("i")));
		}
		else if (eName.equals(Constants.REDUC_VARIABLE)) {
			currentChainVar.add(Integer.parseInt(attrs.getValue("i")));
		}
	}

	public void endElement(String namespaceURI, String sName, String qName) throws SAXException {

		String eName = sName;

		if ("".equals(eName))
			eName = qName;

		if (eName.equals(Constants.ROOT)) {
			// sorting elements before storing 
			Collections.sort(currentRootClause);
			
			// add to proof tree
			Peripherality.proofTree.add(currentRootIndex, new Clause(currentRootClause, currentRootMP, null, null, -1, currentChildren, currentRootIndex));
			Peripherality.clauseOrder.add(currentRootIndex);
		}
		if (eName.equals(Constants.DERIVED)) {
			generateIntermediateClauses(currentChainId, currentChainVar, currentDerivedIndex);
		}

	}

	public void generateIntermediateClauses(ArrayList<Integer> chainId, ArrayList<Integer> chainVar, int derivedIndex) {

		Clause left = Peripherality.proofTree.get(chainId.get(0));
		Clause inter = new Clause();
		Clause right;
		int variable;

		int size = chainId.size();
		for (int i = 1; i < size; i++) {
			right = Peripherality.proofTree.get(chainId.get(i));
			variable = chainVar.get(i - 1);

			inter = new Clause();

			inter.variables = new ArrayList<Integer>(left.variables);
			int size2 = right.variables.size(); 
			for (int j = 0; j < size2; j++) {
				if (!inter.variables.contains(right.variables.get(j)))
					inter.variables.add(right.variables.get(j));
			}

			inter.variables.remove((Integer) variable);
			inter.variables.remove((Integer) (-1 * variable));

			inter.leftParent = left;
			inter.rightParent = right;
			inter.reductionVariable = variable;

			if (i == (chainId.size() - 1)) {
				inter.unprocessedChildren = currentChildren;
				inter.index = derivedIndex;
				Peripherality.proofTree.add(derivedIndex, inter);
				Peripherality.clauseOrder.add(derivedIndex);
			} else{
				inter.unprocessedChildren = 1;
				inter.index = ++lastIndex;
				Peripherality.proofTree.add(inter);
				Peripherality.clauseOrder.add(inter.index);
			}

			left = inter;
		}

	}

}
