package edu.toronto.cs.parsers;

import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import edu.toronto.cs.ProofPruner;
import edu.toronto.cs.util.Constants;

public class ProofPrunerParser extends DefaultHandler {

	private static ArrayList<Integer> clauseIndexes;
	private static PrintWriter out; 
	private String outputFilename;

	private static boolean sign;
	private static int index;

	private static ArrayList<Integer> currentRootClause;
	private static int currentRootClauseIndex;

	private static ArrayList<Integer> currentParents;
	private static ArrayList<Integer> currentResolVariables;
	private static int currentDerivedClauseIndex;

	private boolean emptyClause = true;
	private boolean secondPass = false;
	
	public ProofPrunerParser(String outputFilename, boolean secondPass){
		super();
		this.outputFilename = outputFilename;
		this.secondPass = secondPass;
	}
	
	public void startElement(String namespaceURI, String sName, String qName, Attributes attrs) throws SAXException {
		String eName = sName;

		if ("".equals(eName))
			eName = qName;

		if (eName.equals("proof")) {
			clauseIndexes = new ArrayList<Integer>(10000);

			try {
				out = new PrintWriter(new FileWriter(outputFilename));
			}
			catch (Exception e) {
			}
			out.println("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n<proof>");
		}
		if (eName.equals(Constants.ROOT)) {
			index = Integer.parseInt(attrs.getValue("i"));
			
			// A proof node has this clause as its' parent => save this clause to the new file
			if (clauseIndexes.contains(index)){
				currentRootClause = new ArrayList<Integer>();
				currentRootClauseIndex = index;
			}
			else {
				currentRootClause = null;
				currentRootClauseIndex = -1;
			}
			
		} else if (eName.equals(Constants.VARIABLE) && currentRootClauseIndex != -1) {
				sign = Boolean.parseBoolean(attrs.getValue("s"));
				index = Integer.parseInt(attrs.getValue("i"));

				currentRootClause.add(sign ? (-1 * index) : index);

		} else if (eName.equals(Constants.DERIVED)) {
			index = Integer.parseInt(attrs.getValue("i"));

			// A proof node has this clause as its' parent => save this clause to the new file
			if (clauseIndexes.contains(index)){
				currentParents = new ArrayList<Integer>();
				currentResolVariables = new ArrayList<Integer>();
				currentDerivedClauseIndex = index;
			}
			else {
				currentParents = null;
				currentResolVariables = null;
				currentDerivedClauseIndex = -1;
			}

			if (emptyClause){
				emptyClause=!emptyClause;
				currentParents = new ArrayList<Integer>();
				currentResolVariables = new ArrayList<Integer>();
				currentDerivedClauseIndex = index;
			}

		} else if (eName.equals(Constants.PARENT) && currentDerivedClauseIndex != -1) {
				currentParents.add(Integer.parseInt(attrs.getValue("i")));
		} else if (eName.equals(Constants.REDUC_VARIABLE) && currentDerivedClauseIndex != -1) {
				currentResolVariables.add(Integer.parseInt(attrs.getValue("i")));
		}
	}

	public void endElement(String namespaceURI, String sName, String qName) throws SAXException {

		String eName = sName;

		if ("".equals(eName))
			eName = qName;

		if (eName.equals("proof")) {
			out.println("</proof>");
			out.close();
		}
		else if (eName.equals(Constants.ROOT) && currentRootClauseIndex != -1) {
			// sorting elements before storing (makes retrieval easier when
			// matching against model and property clause files.
			Collections.sort(currentRootClause);
			
			// save this root clause
				StringBuffer node = new StringBuffer("<r i=\"" + currentRootClauseIndex + "\"");

				if (secondPass){
					// save model and property info
					node.append(" o=\"" + ProofPruner.rootClauseLabels.get(currentRootClause.toString()) + "\"" 
							+ " c=\"" + ProofPruner.clauseChildren.get(currentRootClauseIndex) + "\"");
				}

				node.append(">");
				
				int size = currentRootClause.size();
				for (int i = 0; i < size; i++){
					index = currentRootClause.get(i);
					sign = false;
					
					if (index < 0){
						index*=-1;
						sign = true;
					}
					
					node.append("<v i=\"" + index + "\" s=\"" + sign + "\" />");
				}
				
				node.append("</r>");
				out.println(node);
				
				if (!secondPass)
					ProofPruner.rootClauses.add(currentRootClause.toString());
				
				
		}
		else if (eName.equals(Constants.DERIVED) && currentDerivedClauseIndex != -1) {
			// save this derived clause
				StringBuffer node = new StringBuffer("<d i=\"" + currentDerivedClauseIndex + "\"");
				if (secondPass){
					Integer la = ProofPruner.clauseChildren.get(currentDerivedClauseIndex);
					if (la == null)
						node.append(" c=\"0\"");
					else
						node.append(" c=\"" + la + "\"");
						
				}
				node.append(">");

				clauseIndexes.remove((Integer)currentDerivedClauseIndex);
				
				// first write out parents
				int size = currentParents.size();
				for (int i = 0; i < size; i++){
					index = currentParents.get(i);
					node.append("<p i=\"" + index + "\" />");
					if (!clauseIndexes.contains(index))
						clauseIndexes.add(index);

					if (!secondPass){
						int ind = 0;
						try {
							ind = ProofPruner.clauseChildren.get(index);
						}
						catch(Exception e){	}
						ProofPruner.clauseChildren.put(index, ++ind);
					}
				}

				// now write out resolution variables
				size = currentResolVariables.size();
				for (int i = 0; i < size; i++)
					node.append("<rv i=\"" + currentResolVariables.get(i) + "\" />");

				node.append("</d>");
				out.println(node);
		}
	}
}
