package edu.toronto.cs.parsers;

import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import edu.toronto.cs.util.Constants;

public class ProofReindexParser extends DefaultHandler {

	private static HashMap<Integer, Integer> clauseIndexMapping;
	private static int clauseCounter = 0;
	
	private static PrintWriter out; 
	private String outputFilename;

	private static boolean sign;
	private static int index;

	private static ArrayList<Integer> currentRootClause;
	private static int currentRootClauseIndex;
	private static int currentRootMP;
	private static int currentChildren;

	private static ArrayList<Integer> currentParents;
	private static ArrayList<Integer> currentResolVariables;
	private static int currentDerivedClauseIndex;

	public ProofReindexParser(String outputFilename){
		super();
		this.outputFilename = outputFilename;
	}
	
	public void startElement(String namespaceURI, String sName, String qName, Attributes attrs) throws SAXException {
		String eName = sName;

		if ("".equals(eName))
			eName = qName;

		if (eName.equals("proof")) {
			clauseIndexMapping = new HashMap<Integer, Integer>();

			try {
				out = new PrintWriter(new FileWriter(outputFilename));
			}
			catch (Exception e) {
			}
			out.println("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n<proof>");
		}

		if (eName.equals(Constants.ROOT)) {
			currentRootClauseIndex = Integer.parseInt(attrs.getValue("i"));
			clauseIndexMapping.put(currentRootClauseIndex, clauseCounter++);
			
			try{
				currentRootMP = Integer.parseInt(attrs.getValue("o"));
				currentChildren = Integer.parseInt(attrs.getValue("c"));
			}
			catch(Exception E){
				currentRootMP = -1;
			}

			currentRootClause = new ArrayList<Integer>();

		} else if (eName.equals(Constants.VARIABLE)) {
			sign = Boolean.parseBoolean(attrs.getValue("s"));
			index = Integer.parseInt(attrs.getValue("i"));

			currentRootClause.add(sign ? (-1 * index) : index);
		}
		else if (eName.equals(Constants.DERIVED)) {
			currentDerivedClauseIndex = Integer.parseInt(attrs.getValue("i"));
			currentChildren = Integer.parseInt(attrs.getValue("c"));
			clauseIndexMapping.put(currentDerivedClauseIndex, clauseCounter++);
			
			currentParents = new ArrayList<Integer>();
			currentResolVariables = new ArrayList<Integer>();
			
		}
		else if (eName.equals(Constants.PARENT)) {
			currentParents.add(Integer.parseInt(attrs.getValue("i")));
		}
		else if (eName.equals(Constants.REDUC_VARIABLE)) {
			currentResolVariables.add(Integer.parseInt(attrs.getValue("i")));
		}
		
	}

	public void endElement(String namespaceURI, String sName, String qName) throws SAXException {

		String eName = sName;

		if ("".equals(eName))
			eName = qName;

		if (eName.equals("proof")) {
			out.println("</proof>");
			out.close();
		}
		else if (eName.equals(Constants.ROOT)) {
			// sorting elements before storing (makes retrieval easier when
			// matching against model and property clause files.
			Collections.sort(currentRootClause);
			
			// save this root clause
				StringBuffer node = new StringBuffer("<r i=\"" + clauseIndexMapping.get(currentRootClauseIndex) + "\"");

				if (currentRootMP != -1){
					// save model and property info
					node.append(" o=\"" + currentRootMP + "\" c=\"" + currentChildren + "\"");
				}

				node.append(">");
				
				int size = currentRootClause.size();
				for (int i = 0; i < size; i++){
					index = currentRootClause.get(i);
					sign = false;
					
					if (index < 0){
						index*=-1;
						sign = true;
					}
					
					node.append("<v i=\"" + index + "\" s=\"" + sign + "\" />");
				}
				
				node.append("</r>");
				out.println(node);
				
		}
		else if (eName.equals(Constants.DERIVED)) {
			// save this derived clause
				StringBuffer node = new StringBuffer("<d i=\"" + clauseIndexMapping.get(currentDerivedClauseIndex) + "\""
						+ " c=\"" + currentChildren + "\">");

				// first write out parents
				int size = currentParents.size();
				for (int i = 0; i < size; i++)
					node.append("<p i=\"" + clauseIndexMapping.get(currentParents.get(i)) + "\" />");

				// now write out resolution variables
				size = currentResolVariables.size();
				for (int i = 0; i < size; i++)
					node.append("<rv i=\"" + currentResolVariables.get(i) + "\" />");

				node.append("</d>");
				out.println(node);
		}
	}
}
